<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class ManutencaoPrevisaoPassageiros extends \Teknisa\GER\Controllers\TeknisaCrud {

    const DATASOURCE_LOG          = "logprevpaxclaprod";
    const DATASOURCE_CLA          = "prevpaxclaprodm";
    const DATASOURCE_CHECKLIST    = "pcp00013_checklists_gerados";
    const DATASOURCE_COMPL        = "prevpaxgerarcompl";
    const DATASOURCE_PEDIDOS      = "prevpaxservprodm";
    const DATASOURCE_PEDIDOS_LOG  = "hprevpaxservprodm";

    protected $dataSourceName = "";

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $MessagesProcedureInflightor, $entityManager, $newCode) {
        $this->crudService                 = $crudService;
        $this->parameterBag                = $parameterBag;
        $this->environment                 = $environment;
        $this->dataSourceManager           = $dataSourceManager;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->entityManager               = $entityManager;
        $this->newCode                     = $newCode;
    }

    public function saveEditClass(DTO\Request\DataSet $request, DTO\Response $response){
        $data = $request->getDataSet();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                //Save in prevpaxclaprodm
                $rows = $request->getDataSet()->getRows();
                $validation = array_pop($rows);
                $rowsClasse = $this->getClassesValidas($rows);
                if(!empty($rowsClasse) && count($rowsClasse)){
                    $this->dataSourceName = self::DATASOURCE_CLA; 
                    $dataSet = new DataSet($this->dataSourceName, $rowsClasse);
                    $requestSave = new DTO\Request\DataSet($dataSet, "", "", "");
                    $rowsSave = array();
                    $rowsSave = $this->crudService->getRowsToSave($requestSave, $this->dataSourceName);
                    $this->crudService->save($this->dataSourceName, $rowsSave);
                   
                    //if success save the old register in log
                    $this->dataSourceName = self::DATASOURCE_LOG;
                    $request_log = $this->createRequest($requestSave, 'LOGPREVPAXCLAPROD', 'NRSEQUENCIAL');
                    $rowsSave = array();
                    $rowsSave = $this->crudService->getRowsToSave($request_log, $this->dataSourceName);
                    $this->crudService->save($this->dataSourceName, $rowsSave);
                   
                    //Validates the need to generate Checklists Complementares
                    if(!empty($validation) && $validation['VALIDAR_CHECKLIST']){
                        $dateChangedChecklist = $this->validateChecklist($validation, $request, $response);
                        $response->addDataSet(new DataSet("checklistsafetados", $dateChangedChecklist));
                    }
                }
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
    }
    public function saveEditPedidos(DTO\Request\DataSet $request, DTO\Response $response){
        $data = $request->getDataSet();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
                //Save in prevpaxclaprodm
                $rows = $request->getDataSet()->getRows();
                $validation = array_pop($rows);
                $this->dataSourceName = self::DATASOURCE_PEDIDOS; 
                $dataSet = new DataSet($this->dataSourceName, $rows);
                $requestSave = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsSave = array();
                $rowsSave = $this->crudService->getRowsToSave($requestSave, $this->dataSourceName);
                //Salvo na tabela de Itens do Pedido
                $this->crudService->save($this->dataSourceName, $rowsSave);
                //Salvar Historico da alteração   
                $this->dataSourceName = self::DATASOURCE_PEDIDOS_LOG;
                $request_pedidos_log = $this->createRequest($requestSave, 'HPREVPAXSERVPRODM', 'NRHPREVPAXSERVPRODM');
                $rowsSave = array();
                $rowsSave = $this->crudService->getRowsToSave($request_pedidos_log, $this->dataSourceName);
                $this->crudService->save($this->dataSourceName, $rowsSave);
                
                //Validates the need to generate Checklists Complementares
                if(!empty($validation) && $validation['VALIDAR_CHECKLIST']){
                    $dateChangedChecklist = $this->validateChecklist($validation, $request, $response);
                    $response->addDataSet(new DataSet("checklistsafetados", $dateChangedChecklist));
                } 
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }  
    }
    private function createRequest (DTO\Request\DataSet $request, $tableName, $primaryColumn){
        $newDataSet = $this->createSequencialLog($request->getDataSet()->getRows(), $tableName, $primaryColumn);
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
        return $newRequest;
    }
    private function createSequencialLog($data, $tableName, $primaryColumn){
        foreach ($data as &$row) {
            $nrSequencial = $this->newCode->getCodeByTableName($tableName, $this->environment->getNrOrgTrab(), 1, 5, false);
            $row[$primaryColumn] = $nrSequencial;
            $row['__is_new'] = true;
        }
        return new DataSet(strtolower($tableName), $data);
    }
    private function validateChecklist($validation, DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $checklistsGerados = $this->getChecklistsGerados($rows[0], $validation);
        $this->saveChecklistCompl($checklistsGerados, $rows, $response);
        $arrayMessages = array();
        $flag = 0;
        $dataChecklist =  $this->unique_multidim_array($checklistsGerados, 'DTPRODUCAOINF');
        foreach ($dataChecklist as $checklist) {
            if($flag == 0){
                array_push($arrayMessages, "PCP00013_NECE_CKL_COMPL");
                $flag = 1;
            }
            array_push($arrayMessages, "PCP00013_NECE_CKL_COMPL_DATA" . "#_#" . substr($checklist['DTPRODUCAOINF'], 0, 10));
        }
        return $arrayMessages;
    }


    private function getComplementares($row) {
        $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array (
                "columnName" => "NRPREVISAOPROD",
                "operator" => "=",
                "value" => $row['NRPREVISAOPROD']
            ),
            array (
                "columnName" => "DTGERARCOMPL",
                "operator" => "=",
                "value" => $row['DTGERARCOMPL']
            ),
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            ),
        );
        
        $dataSet = $this->crudService->getFilteredDataSet(self::DATASOURCE_COMPL, $conditions);
        if(empty($dataSet->getRows()))
            return [];
        else 
            return $dataSet->getRows();
    }

    private function getClassesValidas($rows) {
        $dataValido = array();
        foreach ($rows as $row) {
            $conditions = array(
                array (
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row['CDFILIAL']
                ),
                array (
                    "columnName" => "NRPREVISAOPROD",
                    "operator" => "=",
                    "value" => $row['NRPREVISAOPROD']
                ),
                array (
                    "columnName" => "CDCIA",
                    "operator" => "=",
                    "value" => $row['CDCIA']
                ),
                array (
                    "columnName" => "NRVOO",
                    "operator" => "=",
                    "value" => $row['NRVOO']
                ),
                array (
                    "columnName" => "CDTRECHO",
                    "operator" => "=",
                    "value" => $row['CDTRECHO']
                ),
                array (
                    "columnName" => "CDCLASSE",
                    "operator" => "=",
                    "value" => $row['CDCLASSE']
                ),
                array (
                    "columnName" => "NRORG",
                    "operator" => "=",
                    "value" => $this->environment->getNrOrg()
                ),
            );
            
            $dataSet = $this->crudService->getFilteredDataSet(self::DATASOURCE_CLA, $conditions);
            $dataSet = $dataSet->getRows();
            if(!empty($dataSet) && count($dataSet)){
                $dataSet[0]['__is_new'] = false;
                $dataSet[0]['changed'] = true;
                $dataSet[0]['QTPAXPREVINF'] = $row['QTPAXPREVINF'];
                array_push($dataValido, $dataSet[0]);
            }
        }
        return $dataValido;
    }
    
    private function getChecklistsGerados($row, $validation) {
        $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array (
                "columnName" => "DTPREVISAOPROD",
                "operator" => "=",
                "value" => $row['DTPREVISAOPROD']
            ),
            array (
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => $row['CDCIA']
            ),
            array (
                "columnName" => "CDCLASSE",
                "operator" => "IN",
                "value" => $validation['CLASSES_MODIFICADAS']
            ),
            array (
                "columnName" => "NRVOO",
                "operator" => "=",
                "value" => $row['NRVOO']
            ),
            array (
                "columnName" => "CDTRECHO",
                "operator" => "=",
                "value" => $row['CDTRECHO']
            ),
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            ),
        );
        $filterCriteria = new FilterCriteria(self::DATASOURCE_CHECKLIST, $conditions, null, null);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $filterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array("CDFILIAL", "DTPREVISAOPROD", "CDCIA", "CDCLASSE", "NRVOO", "CDTRECHO", "NRORG"));

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        if(empty($dataSet->getRows()))
            return [];
        else 
            return $dataSet->getRows();
    }
    private function saveChecklistCompl($checklistsGerados, $rows, $response){
        $data = $this->createDataChecklist($checklistsGerados, $rows);
        $dataSet = new DataSet(self::DATASOURCE_COMPL, $data);
        $requestCompl = new DTO\Request\DataSet($dataSet, "", "", "");
        $this->dataSourceName = self::DATASOURCE_COMPL;
        $rowsSave = $this->crudService->getRowsToSave($requestCompl, $this->dataSourceName);
        $this->crudService->save($this->dataSourceName, $rowsSave);
    }
    private function unique_multidim_array($array, $key) { 
        $temp_array = array(); 
        $i = 0; 
        $key_array = array(); 
        
        foreach($array as $val) { 
            if (!in_array($val[$key], $key_array)) { 
                $key_array[$i] = $val[$key]; 
                $temp_array[$i] = $val; 
            } 
            $i++; 
        } 
        return $temp_array; 
    }
    private function createDataChecklist($checklistsGerados, $rows){
        $rowsSave = array();
        foreach ($checklistsGerados as $date) {
            $row = array(
                    "CDFILIAL" => $rows[0]['CDFILIAL'],
                    "NRPREVISAOPROD" => $rows[0]['NRPREVISAOPROD'],
                    "NRPREVPAXGERARCOMPL" => "",
                    "DTGERARCOMPL" => substr($date['DTPRODUCAOCALC'], 0, 10),
                    "NRORG" =>  $this->environment->getNrOrgTrab(),
                    "__is_new_code" => true,
                    "__is_new" => true,
                    "__new_code_pk" => "PREVPAXGERARCOMPL".$rows[0]['CDFILIAL'],
                    "__table_pk" => "NRPREVPAXGERARCOMPL",
                    "__new_code_length" => 10
            );
            $complemSalvo = $this->getComplementares($row);
            if(empty($complemSalvo)){
                array_push($rowsSave, $row);
            } else if(!empty($complemSalvo) && count($complemSalvo) == 0){
                array_push($rowsSave, $row);
            }
        }
        $dataFilter = $this->unique_multidim_array($rowsSave, 'DTGERARCOMPL');
        return $dataFilter;
    }
}