<?php
namespace Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Report\Strategy\BirtStrategy;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class PCP00044CustoPlanejadoRealizado extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $birtStrategy;
    protected $connection;
    protected $utilLista;
    public function __construct(ReportService $reportService, $environment, $connection, $birtStrategy, $utilLista) {
        $this->environment = $environment;
        $this->connection = $connection;
        $this->birtStrategy = $birtStrategy;
        $this->utilLista = $utilLista;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'PCP00500' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
                array (
                  "DTINICIAL"         => "P_DTINICIAL",
                  "DTFINAL"           => "P_DTFINAL",
                  "NMTIPOCUSTO"       => "P_NMTIPOCUSTO",
                  "DTCUSTO"           => "P_DTCUSTO",
                  "NRORG"             => "P_NRORG",
                  "P_NRCTRLCOMPARATIVOPLAREALINF" => "P_NRCTRLCOMPARATIVOPLAREALINF"
                )
        ),
        'PCP00501' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
                array (
                  "DTINICIAL"         => "P_DTINICIAL",
                  "DTFINAL"           => "P_DTFINAL",
                  "NMTIPOCUSTO"       => "P_NMTIPOCUSTO",
                  "DTCUSTO"           => "P_DTCUSTO",
                  "NRORG"             => "P_NRORG",
                  "P_NRCTRLCOMPARATIVOPLAREALINF" => "P_NRCTRLCOMPARATIVOPLAREALINF"
                )
        ),
        'PCP00502' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
                array (
                  "DTINICIAL"         => "P_DTINICIAL",
                  "DTFINAL"           => "P_DTFINAL",
                  "NMTIPOCUSTO"       => "P_NMTIPOCUSTO",
                  "DTCUSTO"           => "P_DTCUSTO",
                  "NRORG"             => "P_NRORG",
                  "P_NRCTRLCOMPARATIVOPLAREALINF" => "P_NRCTRLCOMPARATIVOPLAREALINF"
                )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        $cdcia = isset($row['T_CIA']) && $row['T_CIA'] === 'T' ? 'T' : $row['CDCIA'];
        $voo = isset($row['T_VOO']) && $row['T_VOO'] === 'T' ? 'T' : $row['VOO']; 
        if(is_array($cdcia)){
            $cdcia = implode('_', $cdcia);
            $cdcia = $this->utilLista->criarListaValores($cdcia);
        }
        if(is_array($voo)){
            $voo = implode('_', $voo);
            $voo = $this->utilLista->criarListaValores($voo);
        }
        $row['CDCIA'] = $cdcia;
        $row['VOO'] = $voo;
        $row['P_NRCTRLCOMPARATIVOPLAREALINF'] = $this->executeProcedure($row);
    }
    
    public function executeProcedure($row){
        $procedure = new StoredProcedure($this->connection, 'GERA_COMPARATIVO_PLAN_REAL_INF');
        $procedure->addParam(new Param('P_CDFILIAL',                    Param::PARAM_INPUT,  $row['CDFILIAL']));
        $procedure->addParam(new Param('P_CIAS',                        Param::PARAM_INPUT,  $row['CDCIA']));
        $procedure->addParam(new Param('P_VOOS',                        Param::PARAM_INPUT,  $row['VOO']));
        $procedure->addParam(new Param('P_DTINICIAL',                   Param::PARAM_INPUT,  $row['DTINICIAL']));
        $procedure->addParam(new Param('P_DTFINAL',                     Param::PARAM_INPUT,  $row['DTFINAL']));
        $procedure->addParam(new Param('P_DTCUSTO',                     Param::PARAM_INPUT,  $row['DTCUSTO']));
        $procedure->addParam(new Param('P_TIPOCUSTO',                   Param::PARAM_INPUT,  $row['CDTIPOCUSTO']));
        $procedure->addParam(new Param('P_CDOPERADOR',                  Param::PARAM_INPUT,  $this->environment->getCdOperador()));
        $procedure->addParam(new Param('P_NRORG',                       Param::PARAM_INPUT,  $this->environment->getNrOrg()));
        $procedure->addParam(new Param('P_NRCTRLCOMPARATIVOPLAREALINF', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
        $result = $procedure->execute();
        if (isset($result['P_NRCTRLCOMPARATIVOPLAREALINF'])) {
            return $result['P_NRCTRLCOMPARATIVOPLAREALINF'];
        }
        return '';
    }
    
}